% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prevplot.R
\name{prevplot}
\alias{prevplot}
\title{Plot observed and expected prevalences for a multi-state model.}
\usage{
prevplot(x, prev.obj, M = FALSE, exacttimes = TRUE, ci = FALSE,
  grid = 100L, x.lab.grid = 500L, xlab = "Time",
  ylab = "Prevalence (\%)", lty.fit = 1, lwd.fit = 1, col.fit = "red",
  lty.ci.fit = 2, lwd.ci.fit = 1, col.ci.fit = col.fit, lwd.obs = 1,
  lty.obs = 1, col.obs = "darkblue", legend.pos = "topright",
  par.col = 3, plot.width = 10, plot.height = 5, max.m = 0.1,
  devnew = TRUE, verbose = TRUE)
}
\arguments{
\item{x}{A \code{msm} object.}

\item{prev.obj}{A list computed by \code{\link[msm]{prevalence.msm}}. It can be with or without
confidence intervals. \code{prevplot} will behaves accordingly.}

\item{M}{If \code{TRUE}, then a rough indicator of deviance from the model is computed
(see 'Details'). Default is \code{FALSE}.}

\item{exacttimes}{If \code{TRUE} (default) then transition times are known and exact. This
is inherited from \code{msm} and should be set the same way.}

\item{ci}{If \code{TRUE}, then confidence intervals, if they exist, are plotted.
Default is \code{FALSE}.}

\item{grid}{Define how many points should be used to build the \emph{x} axis. Defaul is 100.}

\item{x.lab.grid}{Define the interval on the \emph{x} axis at which draw tick marks. Default
is 500.}

\item{xlab}{\emph{x} axis label.}

\item{ylab}{\emph{y} axis label.}

\item{lty.fit}{Line type for the expected prevalences. See \code{\link[graphics]{par}}.}

\item{lwd.fit}{Line width for the expected prevalences. See \code{\link[graphics]{par}}.}

\item{col.fit}{Line color for the expected prevalences. See \code{\link[graphics]{par}}.}

\item{lty.ci.fit}{Line type for the expected prevalences confidence limits.
See \code{\link[graphics]{par}}.}

\item{lwd.ci.fit}{Line width for the expected prevalences confidence limits.
See \code{\link[graphics]{par}}.}

\item{col.ci.fit}{Line color for the expected prevalences confidence limits.
See \code{\link[graphics]{par}}.}

\item{lwd.obs}{Line width for the observed prevalences. See \code{\link[graphics]{par}}.}

\item{lty.obs}{Line type for the observed prevalences. See \code{\link[graphics]{par}}.}

\item{col.obs}{Line color for the observed prevalences. See \code{\link[graphics]{par}}.}

\item{legend.pos}{Where to position the legend. Default is \code{"topright"}, but \emph{x} and
\emph{y} coordinate can be passed. If \code{NULL}, then legend is not shown.}

\item{par.col}{The number of columns of the plot. Default is 3.}

\item{plot.width}{Width of new graphical device. Default is 7. See \code{\link[graphics]{par}}.}

\item{plot.height}{Height of new graphical device. Default is 7. See \code{\link[graphics]{par}}.}

\item{max.m}{If \code{M = TRUE}, it adjusts the upper \emph{y} limit when plotting M.}

\item{devnew}{Set the graphical device where to plot. By default, \code{prevplot} plots on a new
device by setting \code{dev.new}. If \code{FALSE}, then a plot is drawn onto the current device
as specified by \code{dev.cur}. If \code{FALSE} and no external devices are opened, then
a plot is drawn using internal graphics. See \code{\link[grDevices]{dev}}.}

\item{verbose}{If \code{FALSE}, all information produced by \code{print}, \code{cat} and
\code{message} are suppressed. All is done internally so that no global
options are changed. \code{verbose} can be set to \code{FALSE} on all common OS
(see also \code{\link[base]{sink}} and \code{\link[base]{options}}). Default is \code{TRUE}.}
}
\description{
Provides a graphical indication of goodness of fit of a multi-state model computed by
\code{\link[msm]{msm}}. It also computes a rough indicator of where the data depart from the
fitted Markov model.
}
\details{
When \code{M = TRUE}, a rough indicator of the deviance from the Markov model is
computed according to Titman and Sharples (2008). A comparison at a given time \eqn{t_i} of a
patient \emph{k} in the state \emph{s} between observed counts \eqn{O_{is}}
with expected ones \eqn{E_{is}} is build as follows:
\deqn{M_{is} = \frac{(O_{is} - E_{is})^2}{E_{is}}}{ (O_{is} - E_{is})^2 / E_{is} }
}
\examples{
\dontrun{
data( hosp )

# augmenting the data
hosp_augmented = augment( data = hosp, data_key = subj, n_events = adm_number, pattern = label_3,
                          t_start = dateIN, t_end = dateOUT, t_cens = dateCENS )

# let's define the initial transition matrix for our model
Qmat = matrix( data = 0, nrow = 3, ncol = 3, byrow = TRUE )
Qmat[ 1, 1:3 ] = 1
Qmat[ 2, 1:3 ] = 1
colnames( Qmat ) = c( 'IN', 'OUT', 'DEAD' )
rownames( Qmat ) = c( 'IN', 'OUT', 'DEAD' )

# attaching the msm package and running the model using
# gender and age as covariates
library( msm )
msm_model = msm( status_num ~ augmented_int,
                 subject = subj, data = hosp_augmented, covariates = ~ gender + age,
                 exacttimes = TRUE, gen.inits = TRUE, qmatrix = Qmat, method = 'BFGS',
                 control = list( fnscale = 6e+05, trace = 0,
                 REPORT = 1, maxit = 10000 ) )

# defining the times at which compute the prevalences
t_min = min( hosp_augmented$augmented_int )
t_max = max( hosp_augmented$augmented_int )
steps = 100L

# computing prevalences
prev = prevalence.msm( msm_model, covariates = 'mean', ci = 'normal',
                       times = seq( t_min, t_max, steps ) )

# and plotting them using prevplot()
prevplot( msm_model, prev, ci = TRUE, devnew = FALSE, verbose = FALSE )
}

}
\author{
Francesco Grossetti \email{francesco.grossetti@polimi.it}.
}
\references{
Titman, A. and Sharples, L.D. (2010). Model diagnostics for multi-state models,
\emph{Statistical Methods in Medical Research}, 19, 621-651.\cr

Titman, A. and Sharples, L.D. (2008). A general goodness-of-fit test for Markov and
hidden Markov models, \emph{Statistics in Medicine}, 27, 2177-2195. \cr

Jackson, C.H. (2011). Multi-State Models for Panel Data:
The \emph{msm} Package for R. Journal of Statistical Software, 38(8), 1-29.
URL \url{http://www.jstatsoft.org/v38/i08/}.
}
\seealso{
\code{\link[msm]{plot.prevalence.msm}}, \code{\link[msm]{msm}},
\code{\link[msm]{prevalence.msm}}
}

